/******************************************************************************/
//
// Name: 	Interfacing SDRAM with ADSP-21371 Processor
//
/******************************************************************************

(C) Copyright 2005 - Analog Devices, Inc.  All rights reserved.

File Name:		main.asm

Date Modified:	20 Nov 2006 - JJ Rev 1.0

Software:       VisualDSP++4.5

Purpose:	To demonstrate 32-bit SDRAM interface with ADSP-21371 processor	
				
*******************************************************************************/

#include <def21371.h>

#define DataCnt 500 //This is the number of words to be written to the SDRAM and read
					//back from the SDRAM
#define PLLSetlDelay 4096 //Delay needed to settle PLL

// Macros to use different combinations of core clock to sdram clock ratio
// Uncomment ONLY ONE of the following combinations
#define CCLK_SDCLK_RATIO_2_0 // SDCLK = 133 MHz for a core clock of 266 MHz

/* For other ratios, the value of RDIV should be re-calculated. Also, the SDCTL
   configuration value would change */
   
//#define CCLK_SDCLK_RATIO_2_5 
//#define CCLK_SDCLK_RATIO_3_0
//#define CCLK_SDCLK_RATIO_3_5
//#define CCLK_SDCLK_RATIO_4_0

/* Out of the following three write options, un-comment only one at a time */
//#define WRITE_DM_Bus_To_Acess_Int_Ext_Memory //Non-multifunctional instruction
//#define WRITE_OPT_DM
#define WRITE_OPT_PM

/* Out of the following two read options, un-comment only one at a time */ 
#define READ_NO_OPT
//#define READ_OPT

.section/dm seg_dmda;
	.var SourceData[DataCnt];
	.var DestData[DataCnt];
		
.section/pm seg_rth;
	nop;nop;nop;nop;nop;jump start;
	
.section/pm seg_pmco;
start:
	
	CALL Set_CoreCLK_SDRAMClk_Ratio; //Set the Core clock to SDRAM clock ratio
	CALL PLL_Configuration; //Configure PLL to derive specific core clock from clkin
	CALL SDRAM_Bank_Mapping; // Mapping Bank 0 to 32-bit SDRAM
	CALL SDC_Configuration;	 // Configure the processors SDRAM controller - SDC
	CALL SourceDataInit;	 //Initialize Source Data in processors internal SRAM
	CALL Config_Flg_ScopeTrigger;
	
	// dummy READ from SDRAM to start the power up sequence
	nop; nop; nop;
	r13 = dm(0x00200000); //This initiates the power on sequence of SDRAM
	nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;
	nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;
//##################################################################################	
#ifdef WRITE_DM_Bus_To_Acess_Int_Ext_Memory
	bit set FLAGS FLG0; //Drive HI - Trigger LA
	//Write data from SourceData in to the external SDRAM memory
	i1 = SourceData; m1 = 1;
	//Set index to SDRAM starting address in Bank 0
	i2 = 0x00200000; //SDRAM mapped to bank 0 - starting address
	lcntr = DataCnt, do write_sdram_loop until lce;
	r0 = dm(i1,m1);
	write_sdram_loop: dm(i2,m1) = r0;	
#endif	
//#################################################################################


//#################################################################################	
#ifdef WRITE_OPT_DM
	bit set FLAGS FLG0; //Drive HI - Trigger LA
	//Write data from SourceData in to the external SDRAM memory
	i8 = SourceData; m1 = 1;
	
	//Set index to SDRAM starting address in Bank 0
	i1 = 0x00200000; //SDRAM mapped to bank 0 - starting address
	m8 = 1;
	//Perform SDRAM write operations
	r0 = pm(i8,m8);
	lcntr = DataCnt, do write_sdram_loop until lce;
	write_sdram_loop: dm(i1,m1) = r0, r0 = pm(i8,m8);  //Write contents of R0 to SDRAM 
#endif		
//#################################################################################


//#################################################################################
#ifdef WRITE_OPT_PM
	//bit set FLAGS FLG0; //Drive HI - Trigger LA
	//Write data from SourceData in to the external SDRAM memory
	i1 = SourceData; m1 = 1;
	
	//Set index to SDRAM starting address in Bank 0
	i8 = 0x00200000; m8 = 1; //SDRAM mapped to bank 0 - starting address
	r0 = dm(i1,m1);
	lcntr = DataCnt, do write_sdram_loop until lce;
	write_sdram_loop: r0 = dm(i1,m1), pm(i8,m8) = r0; //Write contents of "R0" to SDRAM	
	
#endif
//#################################################################################


	
#ifdef READ_NO_OPT	
	//Read data from external SDRAM in to internal memory array DestData
	i1 = DestData; m1 = 1;
	
	//Set index to SDRAM starting address in Bank 0
	i2 = 0x00200000; //SDRAM mapped to bank 0 - starting address
	
	//Perform SDRAM read operations
	bit set FLAGS FLG0; //Drive HI - Trigger LA
	lcntr = DataCnt, do read_sdram_loop until lce;
	r0 = dm(i2,m1);
	read_sdram_loop: dm(i1,m1) = r0;
#endif
	
#ifdef READ_OPT
	//Read data from external SDRAM in to internal memory array DestData
	i1 = DestData; m1 = 1;
	
	//Set index to SDRAM starting address in Bank 0
	i8 = 0x00200000; m8 = 1;//SDRAM mapped to bank 0 - starting address
	//Perform SDRAM read operations
	bit set FLAGS FLG0; //Drive HI - Trigger LA
	r0 = pm(i8,m8);
	lcntr = DataCnt, do read_sdram_loop until lce;
	read_sdram_loop: dm(i1,m1) = r0, r0 = pm(i8,m8);
#endif

	jump(pc,0);
	
//##################################################################################
//^^^^^^^^^^^^^^^^^^^PLL Configuration subroutine^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
PLL_Configuration:
	
	ustat2 = dm(PMCTL); 
	bit clr ustat2 PLLM63 | PLLD8; //Clear multiplier and divisor bits
	
	bit set ustat2 PLLM10;  /* set a multiplier of 64 and a divider of 3 */ 
	bit set ustat2 PLLBP;   /* Put PLL in bypass mode. */ 
	bit clr ustat2 DIVEN;   /* clear the DIVEN bit  */ 
	dm(PMCTL) = ustat2;   /* The DIVEN bit should be cleared while placing the PLL in bypass mode */ 
	
	//Delay - 
    //wait for PLL to lock at new rate (requirement for modifying multiplier only)
	lcntr = PLLSetlDelay, do pllwait until lce; 
	pllwait: nop; 
	//Delay - Done
	
	ustat2 = dm(PMCTL);     /* DIVEN bit returns as zero */ 
	bit clr ustat2 PLLBP;   /* take PLL out of Bypass, PLL is now at CLKIN*4 (CoreCLK = CLKIN * M/N = CLKIN* 16/4) */ 
	dm(PMCTL) = ustat2;     /* The DIVEN bit should be cleared while taking the PLL out of bypass mode */ 

	RTS;
//##################################################################################


//##################################################################################
//^^^^^^^^^^^^^^^^^^^^^Core clock to SDRAM clock ratio subroutine^^^^^^^^^^^^^^^^^^^	
Set_CoreCLK_SDRAMClk_Ratio:
	// Set the Core clock to SDRAM clock ratio
	ustat1 = dm(PMCTL);
	#ifdef CCLK_SDCLK_RATIO_2_0
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR2 | DIVEN;
	#endif
	#ifdef CCLK_SDCLK_RATIO_2_5
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR2_5 | DIVEN;
	#endif
	#ifdef CCLK_SDCLK_RATIO_3_0
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR3 | DIVEN;
	#endif
	#ifdef CCLK_SDCLK_RATIO_3_5
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR3_5 | DIVEN;
	#endif
	#ifdef CCLK_SDCLK_RATIO_4_0
		bit clr ustat1 SDCKR2 | SDCKR2_5 | SDCKR3 | SDCKR3_5 | SDCKR4;
		bit set ustat1 SDCKR4 | DIVEN;
	#endif
	dm(PMCTL) = ustat1;	
	
	RTS;
//##################################################################################	


//##################################################################################
//^^^^^^^^^^^^^^^^^^^^^^SDRAM Bank Mapping subroutine^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^	
SDRAM_Bank_Mapping:
	// Mapping Bank 0 to 32-bit SDRAM
	// Make sure that jumper is set appropriately so that MS0 is connected to 
	// chip select of 32-bit SDRAM device
	ustat1 = dm(EPCTL);
	bit set ustat1 B0SD;
	bit clr ustat1 B1SD|B2SD|B3SD;
	dm(EPCTL) = ustat1;
	RTS;
//##################################################################################	
	

//##################################################################################
//^^^^^^^^^^^^^^^^^^^^^SDC configuration subroutine^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
SDC_Configuration:
	// Programming SDRAM control registers.
	// RDIV = ((f SDCLK X t REF )/NRA) - (tRAS + tRP )
	ustat1 = 0x814; // (133*(10^6)*64*(10^-3)/4096) - (7+3) 
						// SDRAM clock at 133MHz

	/*For other CCLK_SDCLK ratio values, the RDIV value needs to be re-calculated based
	  on the SDRAM clock frequency. Above formula should be used for to calculate 
	  the RDIV value */
	

	bit set ustat1 SDROPT | BIT_17; // Enabling SDRAM read optimization
										// Setting the Modify to 1
	dm(SDRRC) = ustat1;
	
	ustat1 = 0;
	/*SDCTL value will depend on the SDRAM clock and hence, these values need to be
	  re-calculated for a different SDRAM clock frequency */
	bit set ustat1 SDCL3|SDTRAS6|SDTRP3|SDCAW8|SDPSS|SDTWR2|SDTRCD3|SDRAW12;
	
	dm(SDCTL) = ustat1;
	
	RTS;
//##################################################################################


//##################################################################################
//^^^^^^^^^^^^^^^^^^^^^^^Source data initialization subroutine^^^^^^^^^^^^^^^^^^^^^^
SourceDataInit:
	r0 = 0;
	i1 = SourceData;
	m1 = 1;
	lcntr = DataCnt, do SourceInit until lce;
	dm(i1,m1) = r0;
SourceInit:	r0 = r0 + 1;
	RTS;
//##################################################################################



//##################################################################################
//^^^^^^^^^^^^^^^^^^^^^^^Flg SetUp^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
Config_Flg_ScopeTrigger:
	bit set FLAGS FLG0O;//Configure FLAG0 output
	bit clr FLAGS FLG0; //Drive LOW
	//bit set FLAGS FLG0; //Drive HI
	RTS;



